import '../provider/data.dart';
import '../utils/board_utils.dart';
import '../utils/game_utils.dart';

class GameUtils {

  static Future<void> resetGame(Data myProvider) async {
    myProvider.updateStateRunning = false;
  }

  static Future<void> startGame(Data myProvider) async {
    myProvider.updateSize = myProvider.size;
    myProvider.updateStateRunning = true;
    myProvider.resetGivenTipsCount();
    myProvider.shuffleCellValues();
    myProvider.updateCells = BoardUtils.createEmptyBoard(myProvider.blockSizeHorizontal * myProvider.blockSizeVertical);
    BoardUtils.pickGrid(myProvider);
  }

  static void showTip(Data myProvider) {
    if (myProvider.currentCellCol == null || myProvider.currentCellRow == null) {
      // no selected cell -> pick one
      GameUtils.helpSelectCell(myProvider);
    } else {
      // currently selected cell -> set value
      GameUtils.helpFillCell(myProvider);
    }
    myProvider.increaseGivenTipsCount();
  }

  static void helpSelectCell(Data myProvider) {
    List cells = myProvider.cells;
    int blockSizeHorizontal = myProvider.blockSizeHorizontal;
    int blockSizeVertical = myProvider.blockSizeVertical;

    // pick one of wrong value cells, if found
    List wrongValueCells = BoardUtils.getCellsWithWrongValue(cells, myProvider.cellsSolved, blockSizeHorizontal, blockSizeVertical);
    if (wrongValueCells.length != 0) {
      GameUtils.pickRandomFromList(myProvider, wrongValueCells);
      return;
    }

    // pick one of conflicting cells, if found
    List conflictingCells = BoardUtils.getCellsWithUniqueAvailableValue(cells, blockSizeHorizontal, blockSizeVertical);
    if (conflictingCells.length != 0) {
      GameUtils.pickRandomFromList(myProvider, conflictingCells);
      return;
    }

    //  pick one form cells with unique non-conflicting candidate value
    List candidateCells = BoardUtils.getCellsWithUniqueAvailableValue(cells, blockSizeHorizontal, blockSizeVertical);
    if (candidateCells.length != 0) {
      GameUtils.pickRandomFromList(myProvider, candidateCells);
      return;
    }
  }

  static void pickRandomFromList(Data myProvider, List cellsCoordinates) {
    if (cellsCoordinates.length > 0) {
      cellsCoordinates.shuffle();
      List cell = cellsCoordinates[0];
      myProvider.selectCell(cell[0], cell[1]);
    }
  }

  static void helpFillCell(Data myProvider) {
    List cells = myProvider.cells;
    int blockSizeHorizontal = myProvider.blockSizeHorizontal;
    int blockSizeVertical = myProvider.blockSizeVertical;

    int boardSize = blockSizeHorizontal * blockSizeVertical;

    // Will clean cell if no eligible value found
    int eligibleValue = 0;

    // ensure there is only one eligible value for this cell
    int allowedValuesCount = 0;
    for (var value = 1; value <= boardSize; value++) {
      if (BoardUtils.isValueAllowed(cells, blockSizeHorizontal, blockSizeVertical, myProvider.currentCellCol, myProvider.currentCellRow, value)) {
        allowedValuesCount++;
        eligibleValue = value;
      }
    }

    myProvider.updateCellValue(myProvider.currentCellCol, myProvider.currentCellRow, allowedValuesCount == 1 ? eligibleValue : 0);
    myProvider.selectCell(null, null);
    BoardUtils.computeConflictsInBoard(myProvider);
  }

}
